from abc import ABCMeta, abstractmethod


class Strategy(metaclass=ABCMeta):

    def __init__(self, broker=None, **params):
        self.params = params
        self.broker = broker
        self.init()

    @abstractmethod
    def init(self):
        pass

    @abstractmethod
    def execute(self):
        pass

    def addBroker(self, broker):
        self.broker = broker



import time
import sys
import krakenex
from pykrakenapi import KrakenAPI


class Bot:

    def __init__(self, useKraken=True, interval=1):
        self.useKraken = useKraken
        self.KAPI = KrakenAPI(krakenex.API())
        self.ohlc, self.since = self.KAPI.get_ohlc_data(
            "BTCUSD",
            interval=interval,
            since=None,
            ascending=True,
        )
        self.strategy = []

    def run(self, interval=1):
        wait = 60
        maxRows = 2880
        blockSize = 1440
        retry = 0
        while retry < 10:
            try:
                ohlcnew, self.since = self.KAPI.get_ohlc_data(
                    "BTCUSD",
                    interval=interval,
                    since=self.since,
                    ascending=True,
                )
            except [TimeoutError, ConnectionError]:
                time.sleep(wait)
                retry += 1
                continue

            # Remove the last enty as it is unconfirmed
            self.ohlc.drop(self.ohlc.tail(1).index, inplace=True)
            self.ohlc = self.ohlc.append(ohlcnew)
            if len(self.ohlc) >= maxRows:
                self.ohlc.drop(self.ohlc.head(blockSize).index,
                               inplace=True)
                
            if self.strategy:
                for strat in self.strategy:
                    strat.execute(self.ohlc)
            else:
                raise ValueError("Add Strategy to run Bot")

            sys.stdout.flush()
            time.sleep(interval*wait)

    def addStrategy(self, strategy):
        if strategy.broker is None:
            raise ValueError("Add Broker to Strategy first")
        self.strategy.append(strategy)



from LNMarkets import Positions
from .Notifier import Notifier, addMessage


class Broker:

    def __init__(self, token, initialBalance, silent=False):
        self.token = token
        self.initialBalance = initialBalance
        self.notifier = Notifier(silent)

    @staticmethod
    def calculateProfit(positions):
        totalProfit = 0.0
        for position in positions:
            totalProfit += float(position['pl'])

        return totalProfit

    @staticmethod
    def calculateMargin(positions):
        totalMargin = 0.0
        for position in positions:
            totalMargin += float(position['margin'])
        return totalMargin

    @property
    def openPositions(self):
        return Positions.getPositions(self.token, "open")['open']

    @property
    def closedPositions(self):
        return Positions.getPositions(self.token, "closed")['closed']

    @property
    def unrealizedProfit(self):
        return self.calculateProfit(self.openPositions)

    @property
    def realizedProfit(self):
        return self.calculateProfit(self.closedPositions)

    @property
    def balance(self):
        return self.realizedProfit + self.initialBalance

    @property
    def marginWithheld(self):
        return self.calculateMargin(self.openPositions)

    @addMessage
    def buy(self, leverage, margin=None, quantity=None, stoploss=None,
            takeprofit=None):
        return Positions.buy(
            token=self.token,
            leverage=leverage,
            margin=margin,
            quantity=quantity,
            stoploss=stoploss,
            takeprofit=takeprofit,
        )

    @addMessage
    def limitBuy(self, leverage, price, margin=None, quantity=None,
                 stoploss=None, takeprofit=None):
        return Positions.limitBuy(
            token=self.token,
            leverage=leverage,
            price=price,
            margin=margin,
            quantity=quantity,
            stoploss=stoploss,
            takeprofit=takeprofit,
        )

    @addMessage
    def sell(self, leverage, margin=None, quantity=None, stoploss=None,
             takeprofit=None):
        return Positions.sell(
            token=self.token,
            leverage=leverage,
            margin=margin,
            quantity=quantity,
            stoploss=stoploss,
            takeprofit=takeprofit,
        )

    @addMessage
    def limitSell(self, leverage, price, margin=None, quantity=None,
                  stoploss=None, takeprofit=None):
        return Positions.limitSell(
            token=self.token,
            leverage=leverage,
            price=price,
            margin=margin,
            quantity=quantity,
            stoploss=stoploss,
            takeprofit=takeprofit,
        )

    @addMessage
    def updateProfit(self, pid, price):
        return Positions.updatePosition(
            token=self.token,
            pid=pid,
            type_='takeprofit',
            value=price,
            )

    @addMessage
    def updateStoploss(self, pid, price):
        return Positions.updatePosition(
            token=self.token,
            pid=pid,
            type_='stoploss',
            value=price,
            )

    @addMessage
    def closePosition(self, pid):
        return Positions.closePosition(self.token, pid)

    @addMessage
    def closeAllLongs(self):
        return Positions.closeAllLongs(self.token)

    @addMessage
    def closeAllShorts(self):
        return Positions.closeAllShorts(self.token)

    @addMessage
    def cancelPosition(self, pid):
        return Positions.cancelPosition(self.token, pid)

    @addMessage
    def addMargin(self, pid, amount):
        return Positions.addMargin(self.token, pid, amount)

    @addMessage
    def cashin(self, pid, amount):
        return Positions.cashin(self.token, pid, amount)



import logging
from notifiers import get_notifier
from notifiers.logging import NotificationHandler
from collections import namedtuple


class Notifier:

    def __init__(self, silent=False):
        self.telegram = None
        self.pushover = None
        self.silent = silent

    def enableTelegram(self, chatID, token):
        self.telegram = namedtuple('Telegram', 'notifier chatID token')(
            get_notifier('telegram'),
            chatID,
            token,
            )

        defaults = {
            'chat_id': chatID,
            'token': token,
            }
        handler = NotificationHandler('telegram', defaults=defaults)
        handler.setLevel(logging.ERROR)
        handler.setFormatter(logging.Formatter('%(asctime)s - %(message)s'))
        log = logging.getLogger()
        log.addHandler(handler)
        log.error('Telegram enabled for logging')

    def enablePushover(self, userkey, APIkey):
        self.pushover = namedtuple('Pushover', 'notifier APIkey userkey')(
            get_notifier('pushover'),
            APIkey,
            userkey,
            )

        defaults = {
            'user': userkey,
            'token': APIkey,
            }
        handler = NotificationHandler('pushover', defaults=defaults)
        handler.setLevel(logging.ERROR)
        handler.setFormatter(logging.Formatter('%(asctime)s - %(message)s'))
        log = logging.getLogger()
        log.addHandler(handler)
        log.error('Pushover enabled for logging')

    def notify(self, message):
        if self.telegram is not None:
            self.telegram.notifier.notify(
                token=self.telegram.token,
                chat_id=self.telegram.chatID,
                message=message,
            )
        if self.pushover is not None:
            self.pushover.notifier.notify(
                token=self.pushover.APIkey,
                user=self.pushover.userkey,
                message=message,
            )


def addMessage(f):
    def Wrapper(self, *args, **kwargs):
        message = f'Calling {f.__name__}('
        if args:
            message += ','.join([str(e) for e in args]) + ','
        if kwargs:
            message += str(kwargs)[1:-1]
        message += ')'
        if not self.notifier.silent:
            self.notifier.notify(message)
        return f(self, *args, **kwargs)
    return Wrapper



from setuptools import setup, find_packages
import sys


if sys.version_info < (3, 6):
    raise RuntimeError("This package requres Python 3.6+")
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='LNMarketBot',
    packages=find_packages(include=['LNMarketBot']),
    version='0.1.2',
    description='Trading Bot for LNMarkets',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/DivyanshuBagga/LNMarketBot",
    author='Divyanshu Bagga',
    author_email="divyanshu.baggar@pm.me",
    license='MIT',
    install_requires=[
        'requests',
        'krakenex',
        'pykrakenapi',
        'LNMarkets',
        'notifiers',
    ],
    download_url='https://github.com/DivyanshuBagga/LNMarketBot/archive/'
    '0.1.2.tar.gz',
    keywords=['Bitcoin', 'Finance', 'Trading'],
    python_requires='>=3.6',
)
